@@ -153,11 +153,16 @@ class Client:
             contact: a phone number with format "+X (XXX) XXX-XXXX" or email.
             contactMethod: PHONE or EMAIL.
             description: a group description.
-            userRoles: list who is an admin, editor, and viewer.
+            userRoles: list who is an admin and member.
 
         Returns:
             group: the resulting group or None if error.
         '''
+        if not userRoles:
+            user = self.user()
+            if not user:
+                return
+            userRoles = {'admin': [user['name']], 'member': []}
         return self._gql(q.mutation_createGroup, locals())
 
     def createOauth2Client(
@@ -184,16 +189,8 @@ class Client:
         contactMethod='EMAIL',
         description='',
         isOpen=False,
-        userRoles={
-            'admin': [],
-            'editor': [],
-            'viewer': []
-        },
-        groupRoles={
-            'admin': [],
-            'editor': [],
-            'viewer': []
-        },
+        userRoles=None,
+        groupRoles=None,
         files=[],
     ):
         '''Creates a project.
@@ -212,17 +209,33 @@ class Client:
             project: the resulting project or None if error.
         '''
         variables = {k: v for k, v in locals().items() if k != 'files'}
+        if not userRoles:
+            user = self.user()
+            if not user:
+                return
+            userRoles = {'admin': [user['name']], 'editor': [], 'viewer': []}
+        if not groupRoles:
+            groupRoles = {'admin': [], 'editor': [], 'viewer': []}
         project = self._gql(q.mutation_createProject, variables)
         if not project:
             return
-        self._upload_files(project['id'], files)
+        self.upload_files(project['id'], files)
         return self._gql(q.query_project, {'id': project['id']})
 
-    def _upload_files(self, projectId, files):
+    def upload_files(self, projectId, files):
         with Pool(cpu_count()) as p:
             args = [(self.endpoint, self.token, projectId, f) for f in files]
             p.starmap(_upload_file, args)
 
+    def upload_file(self, projectId, path):
+        '''Upload a file locally to a project.
+
+        Args:
+            projectId: the id of the project.
+            path: the path of the file to upload.
+        '''
+        return self.upload_file(projectId, [path])
+
     def deleteFile(self, projectId, fileName):
         '''Deletes `filename` from `projectId`.'''
         return self._gql(q.mutation_deleteFile, locals())
@@ -336,7 +349,9 @@ class Client:
             return
         d = {k: v for k, v in locals().items() if k != 'self' and v}
         project.update(d)
-        return self._gql(q.mutation_updateOauth2Client, project)
+        project = self._gql(q.mutation_updateProject, project)
+        self.upload_files(project['id'], files)
+        return self._gql(q.query_project, {'id': project['id']})
 
     def updateUser(
         self,
@@ -594,18 +609,22 @@ def _upload_file(endpoint, token, projectId, path):
         return perr(f'invalid path: {path}')
     uri, err = _get_upload_uri(endpoint, token, projectId, path)
     if err:
-        return err
+        return perr(err)
     err = _put(path, uri['uri'])
     if err:
-        return err
+        return perr(err)
 
 
 def _get_upload_uri(endpoint, token, projectId, path):
     fname = os.path.basename(path)
-    data = _gql(endpoint, token, q.mutation_createFileUploadUri, {
-        'projectId': projectId,
-        'fileName': fname
-    })['createFileUploadUri']
+    data, err = _gql(endpoint, token, q.mutation_createFileUploadUri,
+                     {'input': {
+                         'projectId': projectId,
+                         'fileName': fname,
+                     }})
+    if err:
+        return None, err
+    data = data['createFileUploadUri']
     if data['err']:
         return None, data['err']
     return data['ok'], None
